function [x,res_norm,iter,flag,hist]=gmres_hv(A,b,tol,max_it,x,varargin);

% gmres.m solves the linear system Ax=b using the
% GMRES Method without preconditioning.
% input   A        REAL matrix
%         x        REAL initial guess vector
%         b        REAL right hand side vector
%         max_it   INTEGER maximum number of iterations
%         tol      REAL desired reduction of residual norm
%
% output  x        REAL solution vector
%         res_norm REAL residual norm
%         iter     INTEGER number of iterations performed
%         flag     INTEGER: 0 : solution found to tolerance
%                           1 : no convergence given max_it
%         hist     REAL vector of residual reductions
%
% Heinrich Voss, 21.12.2000
%

if nargin < 2
   error('Not enough input arguments.');
end

if nargin < 3
   tol=1e-3;
end

if nargin < 4
   max_it=100;
end

if nargin < 5
   x = zeros(length(b),1);
end


  q(:,1) = b - A*x;
  z(1) = norm(q(:,1));
  init_res = z(1);                      %initial residual norm
  q(:,1) = q(:,1) / z(1);
  hist(1)=1;
  for iter = 1:max_it                   % begin iteration
    q(:,iter+1) = A*q(:,iter);
    for i=1:iter
      h(i,iter) = q(:,i)'*q(:,iter+1);
      q(:,iter+1) = q(:,iter+1) - h(i,iter)*q(:,i);
      end
    h(iter+1,iter) = norm(q(:,iter+1));
    q(:,iter+1) = q(:,iter+1) / h(iter+1,iter);
    for i=1:iter-1
      he = c(i+1)*h(i,iter)+s(i+1)*h(i+1,iter);
      h(i+1,iter) = s(i+1)*h(i,iter) - c(i+1)*h(i+1,iter);
      h(i,iter)=he;
      end
    alpha = sqrt(h(iter,iter)^2+h(iter+1,iter)^2);
    s(iter+1) = h(iter+1,iter) / alpha;
    c(iter+1) = h(iter,iter) / alpha;
    h(iter,iter) = alpha;
    z(iter+1) = s(iter+1)*z(iter);
    z(iter) = c(iter+1)*z(iter);
    res_red = abs(z(iter+1)) / init_res
    hist(iter+1)=res_red;
    if (res_red < tol), break, end                          % converged
    end

  y(iter) = z(iter) / h(iter,iter);
  for i = iter-1:-1:1
    y(i) = z(i);
    for j = i+1:iter
      y(i) = y(i) - y(j)*h(i,j);
      end
    y(i)=y(i)/h(i,i);
    end
  for i=1:iter
    x = x + y(i)*q(:,i);
    end

  res_norm = abs(z(iter+1));
  if (res_red < tol)
    flag = 0;
  else
    flag = 1;end
